@@ -1,7 +1,7 @@
 /* Edge resistance for move/resize operations */
 
 /* 
- * Copyright (C) 2005 Elijah Newren
+ * Copyright (C) 2005, 2006 Elijah Newren
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -187,6 +187,7 @@ find_nearest_position (const GArray        *edges,
   int compare;
   MetaEdge *edge;
   int best, best_dist, i;
+  gboolean edges_align;
 
   /* Initialize mid, edge, & compare in the off change that the array only
    * has one element.
@@ -223,9 +224,7 @@ find_nearest_position (const GArray        *edges,
   /* Start the search at mid */
   edge = g_array_index (edges, MetaEdge*, mid);
   compare = horizontal ? edge->rect.x : edge->rect.y;
-  gboolean edges_align = horizontal ? 
-    meta_rectangle_vert_overlap (&edge->rect, new_rect) :
-    meta_rectangle_horiz_overlap (&edge->rect, new_rect);
+  edges_align = meta_rectangle_edge_aligns (new_rect, edge);
   if (edges_align &&
       (!only_forward || !points_on_same_side (position, compare, old_position)))
     {
@@ -243,7 +242,7 @@ find_nearest_position (const GArray        *edges,
       edge = g_array_index (edges, MetaEdge*, i);
       compare = horizontal ? edge->rect.x : edge->rect.y;
   
-      gboolean edges_align = horizontal ? 
+      edges_align = horizontal ? 
         meta_rectangle_vert_overlap (&edge->rect, new_rect) :
         meta_rectangle_horiz_overlap (&edge->rect, new_rect);
 
@@ -267,7 +266,7 @@ find_nearest_position (const GArray        *edges,
       edge = g_array_index (edges, MetaEdge*, i);
       compare = horizontal ? edge->rect.x : edge->rect.y;
   
-      gboolean edges_align = horizontal ? 
+      edges_align = horizontal ? 
         meta_rectangle_vert_overlap (&edge->rect, new_rect) :
         meta_rectangle_horiz_overlap (&edge->rect, new_rect);
 
@@ -321,6 +320,7 @@ static int
 apply_edge_resistance (MetaWindow                *window,
                        int                        old_pos,
                        int                        new_pos,
+                       const MetaRectangle       *old_rect,
                        const MetaRectangle       *new_rect,
                        GArray                    *edges,
                        ResistanceDataForAnEdge   *resistance_data,
@@ -329,20 +329,18 @@ apply_edge_resistance (MetaWindow                *window,
                        gboolean                   keyboard_op)
 {
   int i, begin, end;
-  gboolean okay_to_clear_keyboard_buildup = FALSE;
-  int      keyboard_buildup_edge = G_MAXINT;
   gboolean increasing = new_pos > old_pos;
   int      increment = increasing ? 1 : -1;
 
-  const int PIXEL_DISTANCE_THRESHOLD_WINDOW   = 16;
-  const int PIXEL_DISTANCE_THRESHOLD_XINERAMA = 32;
-  const int PIXEL_DISTANCE_THRESHOLD_SCREEN   = 32;
+  const int PIXEL_DISTANCE_THRESHOLD_TOWARDS_WINDOW    = 16;
+  const int PIXEL_DISTANCE_THRESHOLD_AWAYFROM_WINDOW   =  0;
+  const int PIXEL_DISTANCE_THRESHOLD_TOWARDS_XINERAMA  = 32;
+  const int PIXEL_DISTANCE_THRESHOLD_AWAYFROM_XINERAMA =  0;
+  const int PIXEL_DISTANCE_THRESHOLD_TOWARDS_SCREEN    = 32;
+  const int PIXEL_DISTANCE_THRESHOLD_AWAYFROM_SCREEN   =  0;
   const int TIMEOUT_RESISTANCE_LENGTH_MS_WINDOW   =   0;
-  const int TIMEOUT_RESISTANCE_LENGTH_MS_XINERAMA = 100;
-  const int TIMEOUT_RESISTANCE_LENGTH_MS_SCREEN   = 750;
-  const int KEYBOARD_BUILDUP_THRESHOLD_WINDOW   = 16;
-  const int KEYBOARD_BUILDUP_THRESHOLD_XINERAMA = 24;
-  const int KEYBOARD_BUILDUP_THRESHOLD_SCREEN   = 32;
+  const int TIMEOUT_RESISTANCE_LENGTH_MS_XINERAMA =   0;
+  const int TIMEOUT_RESISTANCE_LENGTH_MS_SCREEN   =   0;
 
   /* Quit if no movement was specified */
   if (old_pos == new_pos)
@@ -377,9 +375,8 @@ apply_edge_resistance (MetaWindow                *window,
       int       compare = xdir ? edge->rect.x : edge->rect.y;
 
       /* Find out if this edge is relevant */
-      edges_align = xdir ? 
-        meta_rectangle_vert_overlap (&edge->rect, new_rect) :
-        meta_rectangle_horiz_overlap (&edge->rect, new_rect);
+      edges_align = meta_rectangle_edge_aligns (new_rect, edge)  ||
+                    meta_rectangle_edge_aligns (old_rect, edge);
 
       /* Nothing to do unless the edges align */
       if (!edges_align)
@@ -392,65 +389,14 @@ apply_edge_resistance (MetaWindow                *window,
       /* Rest is easier to read if we split on keyboard vs. mouse op */
       if (keyboard_op)
         {
-          /* KEYBOARD ENERGY BUILDUP RESISTANCE: If the user has is moving
-           * fast enough or has already built up enough "energy", then let
-           * the user past the edge, otherwise stop at this edge.  If the
-           * user was previously stopped at this edge, add movement amount
-           * to the built up energy.
-           */
-
-          /* First, determine the amount of the resistance */
-          int resistance = 0;
-          switch (edge->edge_type)
-            {
-            case META_EDGE_WINDOW:
-              resistance = KEYBOARD_BUILDUP_THRESHOLD_WINDOW;
-              break;
-            case META_EDGE_XINERAMA:
-              resistance = KEYBOARD_BUILDUP_THRESHOLD_XINERAMA;
-              break;
-            case META_EDGE_SCREEN:
-              resistance = KEYBOARD_BUILDUP_THRESHOLD_SCREEN;
-              break;
-            }
-
-          /* Clear any previous buildup if we've run into an edge at a
-           * different location than what we were building up on before.
-           * See below for more details where these get set.
-           */
-          if (okay_to_clear_keyboard_buildup &&
-              compare != keyboard_buildup_edge)
-            {
-              okay_to_clear_keyboard_buildup = FALSE;
-              resistance_data->keyboard_buildup = 0;
-            }
-
-          /* Determine the threshold */
-          int threshold = resistance - resistance_data->keyboard_buildup;
-
-          /* See if threshold hasn't been met yet or not */
-          if (ABS (compare - new_pos) < threshold)
-            {
-              if (resistance_data->keyboard_buildup != 0)
-                resistance_data->keyboard_buildup += ABS (new_pos - compare);
-              else
-                resistance_data->keyboard_buildup = 1; /* 0 causes stuckage */
-              return compare;
-            }
-          else
-            {
-              /* It may be the case that there are two windows with edges
-               * at the same location.  If so, the buildup ought to count
-               * towards both edges.  So we just not that it's okay to
-               * clear the buildup once we find an edge at a different
-               * location.
-               */
-              okay_to_clear_keyboard_buildup = TRUE;
-              keyboard_buildup_edge = compare;
-            }
+          if ((old_pos < compare && compare < new_pos) ||
+              (old_pos > compare && compare > new_pos))
+            return compare;
         }
       else /* mouse op */
         {
+          int threshold;
+
           /* INFINITE RESISTANCE for screen edges under certain cases; If
            * the edge is relevant and we're moving towards it and it's a
            * screen edge and infinite resistance has been requested for
@@ -477,12 +423,10 @@ apply_edge_resistance (MetaWindow                *window,
                   timeout_length_ms = TIMEOUT_RESISTANCE_LENGTH_MS_WINDOW;
                   break;
                 case META_EDGE_XINERAMA:
-                  if (window->require_on_single_xinerama)
-                    timeout_length_ms = TIMEOUT_RESISTANCE_LENGTH_MS_XINERAMA;
+                  timeout_length_ms = TIMEOUT_RESISTANCE_LENGTH_MS_XINERAMA;
                   break;
                 case META_EDGE_SCREEN:
-                  if (window->require_fully_onscreen)
-                    timeout_length_ms = TIMEOUT_RESISTANCE_LENGTH_MS_SCREEN;
+                  timeout_length_ms = TIMEOUT_RESISTANCE_LENGTH_MS_SCREEN;
                   break;
                 }
 
@@ -514,17 +458,26 @@ apply_edge_resistance (MetaWindow                *window,
            */
 
           /* First, determine the threshold */
-          int threshold = 0;
+          threshold = 0;
           switch (edge->edge_type)
             {
             case META_EDGE_WINDOW:
-              threshold = PIXEL_DISTANCE_THRESHOLD_WINDOW;
+              if (movement_towards_edge (edge->side_type, increment))
+                threshold = PIXEL_DISTANCE_THRESHOLD_TOWARDS_WINDOW;
+              else
+                threshold = PIXEL_DISTANCE_THRESHOLD_AWAYFROM_WINDOW;
               break;
             case META_EDGE_XINERAMA:
-              threshold = PIXEL_DISTANCE_THRESHOLD_XINERAMA;
+              if (movement_towards_edge (edge->side_type, increment))
+                threshold = PIXEL_DISTANCE_THRESHOLD_TOWARDS_XINERAMA;
+              else
+                threshold = PIXEL_DISTANCE_THRESHOLD_AWAYFROM_XINERAMA;
               break;
             case META_EDGE_SCREEN:
-              threshold = PIXEL_DISTANCE_THRESHOLD_SCREEN;
+              if (movement_towards_edge (edge->side_type, increment))
+                threshold = PIXEL_DISTANCE_THRESHOLD_TOWARDS_SCREEN;
+              else
+                threshold = PIXEL_DISTANCE_THRESHOLD_AWAYFROM_SCREEN;
               break;
             }
 
@@ -536,12 +489,6 @@ apply_edge_resistance (MetaWindow                *window,
       i += increment;
     }
 
-  /* If we didn't run into any new edges in keyboard buildup but had moved
-   * far enough to get past the last one, clear the buildup
-   */
-  if (okay_to_clear_keyboard_buildup && new_pos != keyboard_buildup_edge)
-    resistance_data->keyboard_buildup = 0;
-
   return new_pos;
 }
 
@@ -549,50 +496,21 @@ static int
 apply_edge_snapping (int                  old_pos,
                      int                  new_pos,
                      const MetaRectangle *new_rect,
-                     GArray              *edges1,
-                     GArray              *edges2,
+                     GArray              *edges,
                      gboolean             xdir,
                      gboolean             keyboard_op)
 {
-  int pos1, pos2;
-  int best;
+  int snap_to;
 
   if (old_pos == new_pos)
     return new_pos;
 
-  /* We look at two sets of edges (e.g. left and right) individually
-   * finding the nearest position among each set of edges and then later
-   * finding the better of these two bests.
-   */
-  pos1 = find_nearest_position (edges1,
-                                new_pos,
-                                old_pos,
-                                new_rect,
-                                xdir,
-                                keyboard_op);
-  pos2 = find_nearest_position (edges2,
-                                new_pos,
-                                old_pos,
-                                new_rect,
-                                xdir,
-                                keyboard_op);
-
-  /* For keyboard snapping, ignore either pos1 or pos2 if they aren't in the
-   * right direction.
-   */
-  if (keyboard_op)
-    {
-      if (!points_on_same_side (old_pos, pos1, new_pos))
-        return pos2;
-      if (!points_on_same_side (old_pos, pos2, new_pos))
-        return pos1;
-    }
-
-  /* Find the better of pos1 and pos2 and return it */
-  if (ABS (pos1 - new_pos) < ABS (pos2 - new_pos))
-    best = pos1;
-  else
-    best = pos2;
+  snap_to = find_nearest_position (edges,
+                                   new_pos,
+                                   old_pos,
+                                   new_rect,
+                                   xdir,
+                                   keyboard_op);
 
   /* If mouse snap-moving, the user could easily accidentally move just a
    * couple pixels in a direction they didn't mean to move; so ignore snap
@@ -600,12 +518,12 @@ apply_edge_snapping (int                  old_pos,
    * anyway.
    */
   if (!keyboard_op &&
-      ABS (best - old_pos) >= 8 &&
+      ABS (snap_to - old_pos) >= 8 &&
       ABS (new_pos - old_pos) < 8)
     return old_pos;
   else
-    /* Otherwise, return the best of the snapping positions found */
-    return best;
+    /* Otherwise, return the snapping position found */
+    return snap_to;
 }
 
 /* This function takes the position (including any frame) of the window and
@@ -644,14 +562,12 @@ apply_edge_resistance_to_each_side (MetaDisplay         *display,
                                         BOX_LEFT (*new_outer),
                                         new_outer,
                                         edge_data->left_edges,
-                                        edge_data->right_edges,
                                         TRUE,
                                         keyboard_op);
 
       new_right  = apply_edge_snapping (BOX_RIGHT (*old_outer),
                                         BOX_RIGHT (*new_outer),
                                         new_outer,
-                                        edge_data->left_edges,
                                         edge_data->right_edges,
                                         TRUE,
                                         keyboard_op);
@@ -660,14 +576,12 @@ apply_edge_resistance_to_each_side (MetaDisplay         *display,
                                         BOX_TOP (*new_outer),
                                         new_outer,
                                         edge_data->top_edges,
-                                        edge_data->bottom_edges,
                                         FALSE,
                                         keyboard_op);
 
       new_bottom = apply_edge_snapping (BOX_BOTTOM (*old_outer),
                                         BOX_BOTTOM (*new_outer),
                                         new_outer,
-                                        edge_data->top_edges,
                                         edge_data->bottom_edges,
                                         FALSE,
                                         keyboard_op);
@@ -678,6 +592,7 @@ apply_edge_resistance_to_each_side (MetaDisplay         *display,
       new_left   = apply_edge_resistance (window,
                                           BOX_LEFT (*old_outer),
                                           BOX_LEFT (*new_outer),
+                                          old_outer,
                                           new_outer,
                                           edge_data->left_edges,
                                           &edge_data->left_data,
@@ -687,6 +602,7 @@ apply_edge_resistance_to_each_side (MetaDisplay         *display,
       new_right  = apply_edge_resistance (window,
                                           BOX_RIGHT (*old_outer),
                                           BOX_RIGHT (*new_outer),
+                                          old_outer,
                                           new_outer,
                                           edge_data->right_edges,
                                           &edge_data->right_data,
@@ -696,6 +612,7 @@ apply_edge_resistance_to_each_side (MetaDisplay         *display,
       new_top    = apply_edge_resistance (window,
                                           BOX_TOP (*old_outer),
                                           BOX_TOP (*new_outer),
+                                          old_outer,
                                           new_outer,
                                           edge_data->top_edges,
                                           &edge_data->top_data,
@@ -705,6 +622,7 @@ apply_edge_resistance_to_each_side (MetaDisplay         *display,
       new_bottom = apply_edge_resistance (window,
                                           BOX_BOTTOM (*old_outer),
                                           BOX_BOTTOM (*new_outer),
+                                          old_outer,
                                           new_outer,
                                           edge_data->bottom_edges,
                                           &edge_data->bottom_data,
@@ -726,27 +644,36 @@ apply_edge_resistance_to_each_side (MetaDisplay         *display,
 void
 meta_display_cleanup_edges (MetaDisplay *display)
 {
-  MetaEdgeResistanceData *edge_data = display->grab_edge_resistance_data;
-  g_assert (edge_data != NULL);
   guint i,j;
+  MetaEdgeResistanceData *edge_data = display->grab_edge_resistance_data;
+  GHashTable *edges_to_be_freed;
+
+  g_assert (edge_data != NULL);
 
   /* We first need to clean out any window edges */
+  edges_to_be_freed = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+                                             g_free, NULL);
   for (i = 0; i < 4; i++)
     {
       GArray *tmp = NULL;
+      MetaDirection dir;
       switch (i)
         {
         case 0:
           tmp = edge_data->left_edges;
+          dir = META_DIRECTION_LEFT;
           break;
         case 1:
           tmp = edge_data->right_edges;
+          dir = META_DIRECTION_RIGHT;
           break;
         case 2:
           tmp = edge_data->top_edges;
+          dir = META_DIRECTION_TOP;
           break;
         case 3:
           tmp = edge_data->bottom_edges;
+          dir = META_DIRECTION_BOTTOM;
           break;
         default:
           g_assert_not_reached ();
@@ -755,16 +682,31 @@ meta_display_cleanup_edges (MetaDisplay *display)
       for (j = 0; j < tmp->len; j++)
         {
           MetaEdge *edge = g_array_index (tmp, MetaEdge*, j);
-          if (edge->edge_type == META_EDGE_WINDOW)
-            g_free (edge);
+          if (edge->edge_type == META_EDGE_WINDOW &&
+              edge->side_type == dir)
+            {
+              /* The same edge will appear in two arrays, and we can't free
+               * it yet we still need to compare edge->side_type for the other
+               * array that it is in.  So store it in a hash table for later
+               * freeing.  Could also do this in a simple linked list.
+               */
+              g_hash_table_insert (edges_to_be_freed, edge, edge);
+            }
         }
     }
 
+  /* Now free all the window edges (the key destroy function is g_free) */
+  g_hash_table_destroy (edges_to_be_freed);
+
   /* Now free the arrays and data */
   g_array_free (edge_data->left_edges, TRUE);
   g_array_free (edge_data->right_edges, TRUE);
   g_array_free (edge_data->top_edges, TRUE);
   g_array_free (edge_data->bottom_edges, TRUE);
+  edge_data->left_edges = NULL;
+  edge_data->right_edges = NULL;
+  edge_data->top_edges = NULL;
+  edge_data->bottom_edges = NULL;
 
   /* Cleanup the timeouts */
   if (edge_data->left_data.timeout_setup   &&
@@ -790,7 +732,7 @@ stupid_sort_requiring_extra_pointer_dereference (gconstpointer a,
 {
   const MetaEdge * const *a_edge = a;
   const MetaEdge * const *b_edge = b;
-  return meta_rectangle_edge_cmp (*a_edge, *b_edge);
+  return meta_rectangle_edge_cmp_ignore_type (*a_edge, *b_edge);
 }
 
 static void
@@ -859,24 +801,23 @@ cache_edges (MetaDisplay *display,
   edge_data->left_edges   = g_array_sized_new (FALSE,
                                                FALSE,
                                                sizeof(MetaEdge*),
-                                               num_left);
+                                               num_left + num_right);
   edge_data->right_edges  = g_array_sized_new (FALSE,
                                                FALSE,
                                                sizeof(MetaEdge*),
-                                               num_right);
+                                               num_left + num_right);
   edge_data->top_edges    = g_array_sized_new (FALSE,
                                                FALSE,
                                                sizeof(MetaEdge*),
-                                               num_top);
+                                               num_top + num_bottom);
   edge_data->bottom_edges = g_array_sized_new (FALSE,
                                                FALSE,
                                                sizeof(MetaEdge*),
-                                               num_bottom);
+                                               num_top + num_bottom);
 
   /*
    * 3rd: Add the edges to the arrays
    */
-  num_left = num_right = num_top = num_bottom = 0;
   for (i = 0; i < 3; i++)
     {
       tmp = NULL;
@@ -901,15 +842,13 @@ cache_edges (MetaDisplay *display,
           switch (edge->side_type)
             {
             case META_DIRECTION_LEFT:
-              g_array_append_val (edge_data->left_edges, edge);
-              break;
             case META_DIRECTION_RIGHT:
+              g_array_append_val (edge_data->left_edges, edge);
               g_array_append_val (edge_data->right_edges, edge);
               break;
             case META_DIRECTION_TOP:
-              g_array_append_val (edge_data->top_edges, edge);
-              break;
             case META_DIRECTION_BOTTOM:
+              g_array_append_val (edge_data->top_edges, edge);
               g_array_append_val (edge_data->bottom_edges, edge);
               break;
             default:
@@ -1088,7 +1027,7 @@ meta_display_compute_resistance_and_snapping_edges (MetaDisplay *display)
            * stacking position than this one.
            */
           while (rem_win_stacking && 
-                 stack_position >= (int)rem_win_stacking->data)
+                 stack_position >= GPOINTER_TO_INT (rem_win_stacking->data))
             {
               rem_windows      = rem_windows->next;
               rem_win_stacking = rem_win_stacking->next;
