@@ -3,6 +3,7 @@
 /* 
  * Copyright (C) 2001 Havoc Pennington
  * Copyright (C) 2004 Rob Adams
+ * Copyright (C) 2005 Elijah Newren
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -60,11 +61,12 @@ static MenuItem menuitems[] = {
   { META_MENU_OP_UNABOVE, NULL, TRUE, N_("On _Top") },
   { META_MENU_OP_MOVE, NULL, FALSE, N_("_Move") },
   { META_MENU_OP_RESIZE, NULL, FALSE, N_("_Resize") },
+  { META_MENU_OP_RECOVER, NULL, FALSE, N_("Move Titlebar On_screen") },
   { 0, NULL, FALSE, NULL }, /* separator */
   { META_MENU_OP_DELETE, METACITY_STOCK_DELETE, FALSE, N_("_Close") },
   { META_MENU_OP_WORKSPACES, NULL, FALSE, NULL }, /* separator */
-  { META_MENU_OP_STICK, NULL, FALSE, N_("Put on _All Workspaces") },
-  { META_MENU_OP_UNSTICK, NULL, FALSE,  N_("Only on _This Workspace") },
+  { META_MENU_OP_STICK, NULL, FALSE, N_("_Always on Visible Workspace") },
+  { META_MENU_OP_UNSTICK, NULL, FALSE,  N_("_Only on This Workspace") },
   { META_MENU_OP_MOVE_LEFT, NULL, FALSE, N_("Move to Workspace _Left") },
   { META_MENU_OP_MOVE_RIGHT, NULL, FALSE, N_("Move to Workspace R_ight") },
   { META_MENU_OP_MOVE_UP, NULL, FALSE, N_("Move to Workspace _Up") },
