@@ -815,7 +815,7 @@ char*    meta_theme_replace_constants     (MetaTheme   *theme,
 PangoFontDescription* meta_gtk_widget_get_font_desc        (GtkWidget            *widget,
                                                             double                scale,
 							    const PangoFontDescription *override);
-int                   meta_pango_font_desc_get_text_height (PangoFontDescription *font_desc,
+int                   meta_pango_font_desc_get_text_height (const PangoFontDescription *font_desc,
                                                             PangoContext         *context);
 
 
